@@ -17,8 +17,14 @@
 #' @param padj_method_LRT Character vector of one or more methods for adjusting P-values from
 #'  LRT tests. Defaults to `"BH"`. P-values from LRT tests will always be adjusted
 #'  independently for each LRT term.
+#' @param ihw_covar Data frame containing covariable(s) used for `IHW::ihw()`. Must contain
+#'  a `"feat_id"` column, matching `dana` object `"feat_id"` labels.
+#' @param ihw_covar_id Character string. Column name in `ihw_covar` used as `covariates`
+#'  argument in `IHW::ihw()`.
+#' @param ihw_args Named list. Additional arguments passed to `IHW::ihw()`. Do not provide
+#'  `covariates` argument, as it will be obtained from `ihw_covar`.
+#' @param storey_args Named list. Additional arguments passed to `qvalue::qvalue()`.
 #' @param verbose Logical. Whether to print informative messages. Defaults to `TRUE`.
-#' @param ... Additional arguments passed to `IHW::ihw()` or `qvalue::qvalue()`.
 #'
 #' @return A modified `dana` object with new columns in the `$fit` and `$lrt` data frames
 #'  for each adjusted P-value method applied (e.g. `padj_BH`, `padj_storey_group`).
@@ -26,7 +32,7 @@
 #' @seealso
 #' * [dana()] for fitting differential analysis models on omics datasets.
 #' * [IHW::ihw()] for inverted hypothesis weighting method details.
-#' * [qvalue::qvalue()] for qvalue method details.
+#' * [qvalue::qvalue()] for Storey's qvalue method details.
 #'
 #' @examples
 #' set.seed(123)
@@ -64,45 +70,64 @@
 #'
 #' # Add adjusted P-values
 #' dana_obj <- dana_obj |>
-#'   adjust_pval(dana_obj,
-#'               padj_method = c("BH", "bonferroni"),
+#'   adjust_pval(padj_method = c("BH", "bonferroni"),
 #'               padj_method_LRT = NULL,
 #'               padj_by = "terms",
 #'               verbose = FALSE)
 #'
 #' @export
 adjust_pval <- function(dana_obj, padj_by = c("all", "terms"),
-                        padj_method = c("BH", "bonferroni", "BY",
-                                        "fdr", "hochberg", "holm",
-                                        "hommel", "IHW", "storey"),
-                        padj_method_LRT = c("BH", "bonferroni", "BY",
-                                            "fdr", "hochberg", "holm",
-                                            "hommel", "IHW", "storey"),
-                        verbose = TRUE, ...) {
+                        padj_method = NULL, padj_method_LRT = NULL,
+                        ihw_covar = NULL, ihw_covar_id = NULL, ihw_args = list(),
+                        storey_args = list(),
+                        verbose = TRUE) {
   # Match or set arguments
   padj_by <- match.arg(padj_by)
-  padj_method <- match.arg(padj_method, several.ok = TRUE)
-  padj_method_LRT <- match.arg(padj_method_LRT, several.ok = TRUE)
+
+  all_methods <- c("BH", "bonferroni", "BY", "fdr", "hochberg", "holm", "hommel",
+                   "IHW", "storey")
+  padj_method <- if (is.null(padj_method)) "BH" else match.arg(padj_method, choices = all_methods, several.ok = TRUE)
+  padj_method_LRT <- if (is.null(padj_method_LRT)) "BH" else match.arg(padj_method_LRT, choices = all_methods, several.ok = TRUE)
 
   # Check parameters
   if (!inherits(dana_obj, "dana")) {
     stop("'dana_obj' must be a 'dana' class object generated by 'readyomics::dana()' function.")
   }
 
-  # Set adjustment method
-  adj_method <- function(i, p_val, ...) {
-    if (i == "IHW") {
-      ihw_obj <- IHW::ihw(p_val, ...)
-      adj_vals <- IHW::adj_pvalues(ihw_obj)
-    } else if (i == "storey") {
-      adj_vals <- try(qvalue::qvalue(p_val, ...)$qvalues)
-      if (inherits(adj_vals, "try-error")) {
-        warning("qvalue failed; falling back to BH adjustment.\n")
-        adj_vals <- stats::p.adjust(p_val, method = "BH")
-      }
-    } else {
-      adj_vals <- stats::p.adjust(p_val, method = i)
+  if (any(padj_method == "IHW", padj_method_LRT == "IHW")) {
+    if (any(is.null(ihw_covar), is.null(ihw_covar_id))) {
+      stop("'ihw_covar' and 'ihw_covar_id' are required when using IHW.")
     }
+    if (!inherits(ihw_covar, "data.frame")) {
+      stop("'ihw_covar' must be a data.frame")
+    } else if (!"feat_id" %in% colnames(ihw_covar)) {
+      stop("'ihw_covar' must contain a column named 'feat_id' matching labels of 'dana' object.")
+    }
+    if (!inherits(ihw_covar_id, "character")) {
+      stop("'ihw_covar_id' must be a character string.")
+    } else if (!ihw_covar_id %in% colnames(ihw_covar)) {
+      stop("'ihw_covar_id' not found in 'ihw_covar' colnames.")
+    }
+  }
+
+  # Set adjustment method
+  adj_method <- function(i, p_val, ihw_args = list(), storey_args = list()) {
+    adj_vals <- try({
+      if (i == "IHW") {
+        ihw_obj <- do.call(IHW::ihw, c(list(pvalues = p_val), ihw_args))
+        adj_vals <- IHW::adj_pvalues(ihw_obj)
+      } else if (i == "storey") {
+        adj_vals <- do.call(qvalue::qvalue, c(list(p = p_val), storey_args))$qvalues
+      } else {
+        adj_vals <- stats::p.adjust(p_val, method = i)
+      }
+    }, silent = FALSE)
+
+    if (inherits(adj_vals, "try-error")) {
+      warning(paste(i, "failed for all or some variables.\n"))
+      adj_vals <- NA
+    }
+
     return(adj_vals)
   }
 
@@ -115,7 +140,16 @@ adjust_pval <- function(dana_obj, padj_by = c("all", "terms"),
     for (i in padj_method) {
       padj_label <- paste("padj", i, sep = "_")
       dana_obj$fit[[padj_label]] <- NA
-      dana_obj$fit[[padj_label]][p_include] <- adj_method(i, p_nominal_include, ...)
+      ihw_args_i <- ihw_args
+      if (i == "IHW") {
+        # Match feature-associated covariable used for IHW to P value
+        ihw_covar_match <- dplyr::left_join(as.data.frame(dana_obj$fit[p_include, "feat_id", drop = FALSE]),
+                                            ihw_covar)
+        ihw_args_i <- utils::modifyList(ihw_args_i, list(covariates = ihw_covar_match[[ihw_covar_id]]))
+      }
+      dana_obj$fit[[padj_label]][p_include] <- adj_method(i, p_nominal_include,
+                                                          ihw_args = ihw_args_i,
+                                                          storey_args = storey_args)
     }
   } else {
     fit_terms <- unique(dana_obj$fit[["Coefficient"]])
@@ -127,7 +161,16 @@ adjust_pval <- function(dana_obj, padj_by = c("all", "terms"),
       for (i in padj_method) {
         padj_label <- paste("padj", i, t, sep = "_")
         dana_obj$fit[[padj_label]] <- NA
-        dana_obj$fit[[padj_label]][p_include] <- adj_method(i, p_nominal_include, ...)
+        ihw_args_i <- ihw_args
+        if (i == "IHW") {
+          # Match feature-associated covariable used for IHW to P value
+          ihw_covar_match <- dplyr::left_join(as.data.frame(dana_obj$fit[p_include, "feat_id", drop = FALSE]),
+                                              ihw_covar)
+          ihw_args_i <- utils::modifyList(ihw_args_i, list(covariates = ihw_covar_match[[ihw_covar_id]]))
+        }
+        dana_obj$fit[[padj_label]][p_include] <- adj_method(i, p_nominal_include,
+                                                            ihw_args = ihw_args_i,
+                                                            storey_args = storey_args)
       }
     }
   }
@@ -145,7 +188,16 @@ adjust_pval <- function(dana_obj, padj_by = c("all", "terms"),
       for (i in padj_method_LRT) {
         padj_label <- paste("padj", i, l, sep = "_")
         dana_obj$lrt[[padj_label]] <- NA
-        dana_obj$lrt[[padj_label]][p_include] <- adj_method(i, p_nominal_lrt_include, ...)
+        ihw_args_i <- ihw_args
+        if (i == "IHW") {
+          # Match feature-associated covariable used for IHW to P value
+          ihw_covar_match <- dplyr::left_join(as.data.frame(dana_obj$lrt[p_include, "feat_id", drop = FALSE]),
+                                              ihw_covar)
+          ihw_args_i <- utils::modifyList(ihw_args_i, list(covariates = ihw_covar_match[[ihw_covar_id]]))
+        }
+        dana_obj$lrt[[padj_label]][p_include] <- adj_method(i, p_nominal_lrt_include,
+                                                            ihw_args = ihw_args_i,
+                                                            storey_args = storey_args)
       }
     }
     # Add LRT P-values to dana_obj$fit table (only for fixed effects)
